/**
 * 
 */
package test.code;

import java.awt.Desktop;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.CellRangeAddress;
import org.apache.poi.hssf.util.CellReference;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

public class StartTest {

	private static final String BLANK_NAME = "/test/blank/blank.xls";

	public static CellReference getNamedCellReference(HSSFWorkbook workbook, String cellName) {
		int namedCellIndex = workbook.getNameIndex(cellName);
		return namedCellIndex < 0 ? null : new CellReference(workbook.getNameAt(namedCellIndex)
				.getReference());
	}

	public static HSSFCell getNamedCell(HSSFWorkbook workbook, String cellName) {
		CellReference cellReference = getNamedCellReference(workbook, cellName);
		if (cellReference == null) {
			return null;
		}
		HSSFSheet sheet = workbook.getSheet(cellReference.getSheetName());
		HSSFRow row = sheet.getRow(cellReference.getRow());
		HSSFCell cell = row.getCell(cellReference.getCol());
		return cell;
	}

	public static void setNamedCellValue(	HSSFWorkbook workbook,
											String cellName,
											HSSFRichTextString value) {
		HSSFCell cell = getNamedCell(workbook, cellName);
		if (cell != null) {
			cell.setCellValue(value);
		}
	}

	public static void setNamedCellValue(HSSFWorkbook workbook, String cellName, String value) {
		setNamedCellValue(workbook, cellName, new HSSFRichTextString(value));
	}

	public static byte[] flushWorkBook(HSSFWorkbook workBook) {
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		try {
			workBook.write(outputStream);
			outputStream.close();
		} catch (IOException e) {
			// process exception
			e.printStackTrace();
		}
		return outputStream.toByteArray();
	}

	public static byte[] fillData() {
		POIFSFileSystem fs;
		HSSFWorkbook workbook;
		try {
			fs = new POIFSFileSystem(StartTest.class.getResourceAsStream(BLANK_NAME));
			workbook = new HSSFWorkbook(fs);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		setNamedCellValue(workbook, "ident", "123456");

		setNamedCellValue(workbook, "dispNum", "123/12.12.12");

		setNamedCellValue(workbook, "index", "A123");
		setNamedCellValue(workbook, "date", "12.12.12");

		setNamedCellValue(workbook, "cipher", "ABCDEF\nABCDEF\nABCDEF");

		setNamedCellValue(workbook, "name", "ABCDEF\nABCDEF\nABCDEF");

		setNamedCellValue(workbook, "desc", "Title");

		setNamedCellValue(workbook, "place", "");

		setNamedCellValue(workbook, "year", "0000");

		setNamedCellValue(workbook, "part", "12");

		setNamedCellValue(workbook, "number", "123456");
		setNamedCellValue(workbook, "fio", "Иванов Иван");

		setNamedCellValue(workbook, "retdate", "12.12.12");
		return flushWorkBook(workbook);
	}

	public static boolean fontsEquals(HSSFFont font1, HSSFFont font2) {
		return (font1.getItalic() == font2.getItalic())
			&& (font1.getStrikeout() == font2.getStrikeout())
			&& (font1.getBoldweight() == font2.getBoldweight())
			&& (font1.getFontHeightInPoints() == font2.getFontHeightInPoints())
			&& (font1.getColor() == font2.getColor())
			&& (font1.getFontName() != null && font1.getFontName().equals(font2.getFontName()))
			&& (font1.getTypeOffset() == font2.getTypeOffset())
			&& (font1.getUnderline() == font2.getUnderline());
	}

	public static HSSFFont copyFontTo(HSSFFont sourceFont, HSSFWorkbook targetWb) {
		if (sourceFont == null) {
			return null;
		}
		for (short i = 0; i < targetWb.getNumberOfFonts(); ++i) {
			HSSFFont font = targetWb.getFontAt(i);
			if (fontsEquals(font, sourceFont)) {
				return font;
			}
		}
		HSSFFont targetFont = targetWb.createFont();
		targetFont.setBoldweight(sourceFont.getBoldweight());
		targetFont.setColor(sourceFont.getColor());
		targetFont.setFontHeightInPoints(sourceFont.getFontHeightInPoints());
		targetFont.setFontName(sourceFont.getFontName());
		targetFont.setItalic(sourceFont.getItalic());
		targetFont.setStrikeout(sourceFont.getStrikeout());
		targetFont.setTypeOffset(sourceFont.getTypeOffset());
		targetFont.setUnderline(sourceFont.getUnderline());
		return targetFont;
	}

	public static boolean styleEquals(	HSSFWorkbook wb1,
										HSSFCellStyle style1,
										HSSFWorkbook wb2,
										HSSFCellStyle style2) {
		return (style1.getAlignment() == style2.getAlignment())
			&& (style1.getBorderBottom() == style2.getBorderBottom())
			&& (style1.getBorderTop() == style2.getBorderTop())
			&& (style1.getBorderLeft() == style2.getBorderLeft())
			&& (style1.getBorderRight() == style2.getBorderRight())
			&& (fontsEquals(style1.getFont(wb1), style2.getFont(wb2)))
			&& (style1.getFillBackgroundColor() == style2.getFillBackgroundColor())
			&& (style1.getFillForegroundColor() == style2.getFillForegroundColor())
			&& (style1.getIndention() == style2.getIndention())
			&& (style1.getRotation() == style2.getRotation())
			&& (style1.getVerticalAlignment() == style2.getVerticalAlignment())
			&& (style1.getWrapText() == style2.getWrapText());
	}

	public static HSSFCellStyle copyStyleTo(HSSFWorkbook sourceWb,
											HSSFCellStyle sourceStyle,
											HSSFWorkbook targetWb) {
		if (sourceStyle == null) {
			return null;
		}
		for (short i = 0; i < targetWb.getNumCellStyles(); ++i) {
			HSSFCellStyle style = targetWb.getCellStyleAt(i);
			if (styleEquals(sourceWb, sourceStyle, targetWb, style)) {
				return style;
			}
		}

		HSSFCellStyle targetStyle = targetWb.createCellStyle();
		targetStyle.setAlignment(sourceStyle.getAlignment());
		targetStyle.setBorderBottom(sourceStyle.getBorderBottom());
		targetStyle.setBorderTop(sourceStyle.getBorderTop());
		targetStyle.setBorderLeft(sourceStyle.getBorderLeft());
		targetStyle.setBorderRight(sourceStyle.getBorderRight());
		HSSFFont targetFont = copyFontTo(sourceStyle.getFont(sourceWb), targetWb);
		if (targetFont != null) {
			targetStyle.setFont(targetFont);
		}
		targetStyle.setFillBackgroundColor(sourceStyle.getFillBackgroundColor());
		targetStyle.setFillForegroundColor(sourceStyle.getFillForegroundColor());
		targetStyle.setIndention(sourceStyle.getIndention());
		targetStyle.setRotation(sourceStyle.getRotation());
		targetStyle.setVerticalAlignment(sourceStyle.getVerticalAlignment());
		targetStyle.setWrapText(sourceStyle.getWrapText());
		return targetStyle;
	}

	public static int getNextUncreatedRowIndex(HSSFSheet sheet, boolean first) {
		return first ? 0 : sheet.getLastRowNum() + 1;
	}

	public static void collectToSingle(	HSSFWorkbook sourceWb,
										HSSFSheet targetSheet,
										HSSFWorkbook targetWb,
										boolean first) {
		if (sourceWb.getNumberOfSheets() > 0) {
			for (int i = 0; i < sourceWb.getNumberOfSheets(); ++i) {
				HSSFSheet sourceSheet = sourceWb.getSheetAt(i);
				int firstRowIndex = getNextUncreatedRowIndex(targetSheet, first);
				for (Iterator<HSSFRow> rowIterator = sourceSheet.rowIterator(); rowIterator
						.hasNext();) {
					HSSFRow sourceRow = rowIterator.next();
					if (sourceRow != null) {
						HSSFRow targetRow =
								targetSheet.createRow(getNextUncreatedRowIndex(targetSheet, first));
						first = false;
						targetRow.setHeightInPoints(sourceRow.getHeightInPoints());
						HSSFCellStyle targetRowStyle =
								copyStyleTo(sourceWb, sourceRow.getRowStyle(), targetWb);
						if (targetRowStyle != null) {
							targetRow.setRowStyle(targetRowStyle);
						}
						for (Iterator<HSSFCell> cellIterator = sourceRow.cellIterator(); cellIterator
								.hasNext();) {
							HSSFCell sourceCell = cellIterator.next();
							if (sourceCell != null) {
								int k = sourceCell.getColumnIndex();
								targetSheet.setColumnWidth(k, sourceSheet.getColumnWidth(k));
								HSSFCell targetCell = targetRow.createCell(k);
								if (sourceCell.getCellComment() != null) {
									targetCell.setCellComment(sourceCell.getCellComment());
								}
								HSSFCellStyle targetCellStyle =
										copyStyleTo(sourceWb, sourceCell.getCellStyle(), targetWb);
								if (targetCellStyle != null) {
									targetCell.setCellStyle(targetCellStyle);
								}
								int type = sourceCell.getCellType();
								targetCell.setCellValue(sourceCell.getRichStringCellValue());
							}
						}
					}
				}
				for (int j = 0; j < sourceSheet.getNumMergedRegions(); j++) {
					CellRangeAddress region = sourceSheet.getMergedRegion(j);
					targetSheet.addMergedRegion(new CellRangeAddress(region.getFirstRow()
						+ firstRowIndex, region.getLastRow() + firstRowIndex, region
							.getFirstColumn(), region.getLastColumn()));
				}
			}
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String fileName = "d:\\1.xls";
		byte[] data = fillData();
		FileOutputStream output;
		HSSFWorkbook target = new HSSFWorkbook();
		HSSFSheet targetSheet = target.createSheet("Лист");
		try {
			collectToSingle(new HSSFWorkbook(new ByteArrayInputStream(data)),
							targetSheet,
							target,
							true);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			return;
		}
		try {
			output = new FileOutputStream(fileName);
			output.write(flushWorkBook(target));
			output.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			Desktop.getDesktop().open(new File(fileName));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
